% va2nafx.m: convert between VA and NAFX.
% Usage: out = va2nafx(va_or_nafx, inputValue, age_range, snel);
%
% where va_or_nafx = 1 converts VA to NAFX
%                  = 2 converts NAFX to VA
% inputValue is the number to be converted.
%
% age_range = 1: under 6 years old
%             2: from 6 to 12 years old
%             3: from 12+ to 40 years old
%             4: from 40+ to 60 years old
%             5: greater than 60 years old
%             6: A dog of any age
%
% snel = 0 returns 'out' as a decimal.
%      = 1 returns 'out' as a Snellen fraction.
%
% Alternatively, you can simply type "va2nafx" and follow the prompts

% Written by:  Jonathan Jacobs
%              May 1998 - October 2004 (last mod: 10/07/04)

function out = va2nafx(va_or_nafx, inpVal, age_range, snel)


if nargin < 4
   snel = 1;
end

if nargin == 0
   mode = 'inter';
   va_or_nafx = -1;
   inpVal=-1;
   verbose=1;
   age_range = -1;
 else
   verbose=0;
   mode = 'batch';
end

if nargout == 1
   verbose = 0;
end

while(age_range<0) | (age_range>7)
disp('What is the age of the subject?')
   disp(' 0) <Quit>')
   disp(' 1) under 6 years')
   disp(' 2) from 6 to 12 years')
   disp(' 3) from 12+ to 40 years')
   disp(' 4) from 40+ to 60 years')
   disp(' 5) older than 60 years ')
   disp(' 6) Woof! A dog (age unimportant).')
   %disp(' 7) Old "young adults" NAFX equation (obsolete).')
   age_range = input(' --> ');
end

switch age_range
case 0
  display('Aborted.')
  return
case 1
  nafx_slope = 0.8; nafx_offset = 0.0;    %% max = 20/25
case {2, 5}
  nafx_slope = 1.0; nafx_offset = 0.0;    %% max = 20/20
case 3
  nafx_slope = 1.375; nafx_offset = 0.0;  %% max = 20/15+
case 4
  nafx_slope = 1.21; nafx_offset = 0.0;   %% max = 20/15-
case 6
  nafx_slope = 0.286; nafx_offset = 0.0;  %% max = 20/70+
case 7
  nafx_slope = 1.44; nafx_offset = 0.065;
end

while(va_or_nafx<1) | (va_or_nafx>2)
disp(' ')
   disp(' 1) Convert VA to NAFX')
   disp(' 2) Convert NAFX to VA')
   va_or_nafx = input(' --> ');
end

switch va_or_nafx
 case 1
   va=inpVal;
   va_lim     = nafx_slope;
   va_lim_str = snellify(va_lim);
   
   disp(' ')
   disp(['Enter the visual acuity (up to ' va_lim_str ,...
         ' = ' num2str(va_lim,'%4.3f') '):' ])
   while (va<0) | (va>va_lim)
      va = input(' --> ');
   end

   nafx = (va+nafx_offset)/nafx_slope;
   if nafx > 1
      disp('Warning: NAFX calculated to be >1.0.  This is not valid.')
      disp('The acuity may be better approximated by a different age')
      disp('population or it may exceed the 50% values for any age population.')
   end

   if verbose
      disp(['  NAFX calculated to be ' num2str(nafx,'%4.3f')])
      return
   end
   out = nafx;


 case 2
   max_nafx = 1/nafx_slope; %% good idea or not to limit input for age group?
   nafx=inpVal;
   if (nafx<0.0 | nafx>1.0) & strcmp(mode,'batch')
      disp(' ')
      disp(['The NAFX is out of range (' num2str(nafx) ').' ,... 
      'It must be between 0.0 and 1.0.'])
      out = NaN;
      return
   end
   while (nafx<0.0 | nafx>1.0)
      disp(' ')
      nafx = input('Enter a NAFX (range 0.0 to 1.0): ');
   end

   va = (nafx*nafx_slope) - nafx_offset;

   % calculate & display snellen strings
   [snelstring1, snelstring2] = snellify(va);
   if verbose, disp(['  VA calculated to be ' num2str(va,'%4.3f')]); end
   if verbose
      disp(['  Snellen Acuity: ' snelstring1 '   (' snelstring2 ')' ])
      return
   end

   if snel
      out = snelstring2;
    else
      out = va;
   end


 otherwise
   ; % shouldn't ever be here
   

end



%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function [snelstring1, snelstring2] = snellify(va);

denom=20/va;

% make pretty snellen output.
basedenom = fix(denom/5)*5;
remainder = denom-basedenom;
if remainder == 0
   sneldenom = basedenom;
   tweak = '';
 elseif remainder < 2.5
   sneldenom = basedenom;
   tweak = '-';
 else
   sneldenom = basedenom+5;
   tweak = '+';
end

snelstring1 = ['20/' num2str(sneldenom) tweak];
snelstring2 = ['20/' num2str(round(denom))   ];
