% edf2bin.m: convert an EDF file to a MATLAB-readable binary formatted file.
% As a default, data is arranged in the following column order: [lh rh lv rv],
% but other configurations (including pupil data) are possible. 
%
% Exporting data from EDF format requires the 'edf2asc' program from SR Research.
% It is freely available to registered users from their support website at:
% https://www.sr-support.com/forums/index.php
% Select the 'EyeLink Display Software' topic and from there download the version
% for your computer's operating system.  Place a copy of the 'edf2asc' executable in
% your OMtools/rd directory to enable this program (edf2bin) to perform the conversion.
%
% This program directly commands edf2asc to export the data to a '.asc' file that has
% the same name as the original .EDF file, and in the same directory.  It is done using
% the following command: edf2asc xxxx.edf -s -y -miss NaN
% Only sample data (i.e., no event information) is exported.  Dropped sample points are
% saved as 'NaN' (the IEEE representation for 'Not a Number').
%
% This program assumes that the data being imported from EDF to MATLAB was recorded 
% binocularly.  If not, you will need to know the channel order so that you can guide
% edf2bin in how to name and arrange the channels.
%

% Written by:  Jonathan Jacobs
%              August 2000 - April 2012 (last mod: 04/09/12)

% 05 Apr 12 -- edf files containing multiple trials will now be properly saved
%              in .bin format             
%              Can now deal with channels in different order than default
% 09 Apr 12 -- Now can directly read .EDF files without requiring user intervention
%              and modifications of intermediate .ASC file.
% 15 Apr 13 -- Fixed for case when EDF time index (column 1) changes from 6 digits to 7

function edf2bin(~)

curdir = pwd;
cd(findomtools)
cd('rd')

if (exist('edf2asc','file') ~= 2)
   disp('To use this version of edf2bin, you need to have a copy of the SR Research')
   disp('executable EyeLink file "edf2asc" installed in the OMtools "rd" directory.')
   disp('If you have registered an account with SR, go to their support website at')
   disp('https://www.sr-support.com/forums/index.php and download the EyeLink Display Software')
   disp('for your platform (Mac, Mac OS X, Windows or Linux.  This will contain the needed file.')
   return
end

try
   cd(curdir)
 catch
   cd(matlabroot)
end 

[fn, pn]=uigetfile({'*.edf'}, 'Select an EDF file to load');
if fn == 0, disp('Aborted.'); return, end
fname = lower(strtok(fn,'.'));
inputfile = ['''' pn fn ''''];
msgsfile = ['''' pn fname '_msgs' ''''];
datafile = ['''' pn fname '_data' ''''];
eventfile = ['''' pn fname '_events' ''''];

cd(findomtools)
cd('rd')
% time for the heavy lifting.

% search the EDF file for sampling frequency and recorded eye channel(s)
% This is what an entry looks like:  MSG	3964147 RECCFG CR 1000 2 1 LR
eval( [ '! ./edf2asc ' inputfile ' ' msgsfile '  -neye -ns -y ' ] )
eval( [ '! ./edf2asc ' inputfile ' ' eventfile '  -nmsg -ns -y ' ] )
disp('EDF messages exported.')
disp('Searching for channel and frequency information.')
% Use carriage return as delimiter. Each line of msgs is a single MSG.
msgs = importdata([pn fname '_msgs.asc'],char(13)); % '13' is CR char.
%assignin('base','msgs', msgs)

%cfglines = zeros(length(10),1); 
ind = 0; ind2 = 0; start_time = [];
for ii = 1:length(msgs)
   st_temp = strfind( msgs{ii}, 'START');
   if st_temp == 1
      ind2 = ind2 + 1;
      [~, temp] = strtok( msgs{ii});
      [start_time{ind2},~] = strtok( temp );
      start_time{ind2} = [start_time{ind2} char(13)];
   end
   
   k=strfind( msgs{ii},'RECCFG' );
   if k~=0
      ind=ind+1;
      %cfglines(ind) = ii; cfgpos(ind)=k; 
      p = strfind( msgs{ii}, '2000'); 
      if p, sf(ind) = 2000; sfpos(ind)= p; end
      p = strfind( msgs{ii}, '1000'); 
      if p, sf(ind) = 1000; sfpos(ind)= p; end
      p = strfind( msgs{ii},  '500'); 
      if p, sf(ind) =  500; sfpos(ind)= p; end
      p = strfind( msgs{ii},  '250'); 
      if p, sf(ind) =  250; sfpos(ind)= p; end
      
      temp = msgs{ii}(sfpos(ind):end);
      [~, pos_type] = strtok( temp );
      [~, eye_code] = strtok( pos_type );
      
      eyes{ind} = 'none';

      if strfind(eye_code, '1'), eyes{ind} = 'l'; end
      if strfind(eye_code, '2'), eyes{ind} = 'r'; end
      if strfind(eye_code, '3'), eyes{ind} = 'lr'; end

      if strfind(eye_code, 'LR'), eyes{ind} = 'lr'; end
      if strfind(eye_code, 'L') & isempty(strfind(eye_code, 'R')), eyes{ind} = 'l'; end
      if strfind(eye_code, 'R') & isempty(strfind(eye_code, 'L')), eyes{ind} = 'r'; end
           
   end
end % for ii

% now we'll export the samples.
eval([ '! ./edf2asc ' inputfile  ' ' datafile ' -s -y -miss NaN' ] )
disp('EDF to ASCII conversion completed.')
disp('Importing converted data into MATLAB.  Patience is a virtue.')
raw = importdata([pn fname '_data' '.asc']);
if isempty(raw)
    disp('No eye movement data found. Aborting')
    return
end

%assignin('base','raw', raw)
cd(curdir)

% chop off everything after the final tab.  This will remove the non-numeric last coluumn.
disp('Data successfully loaded.  Converting to numeric values.  Tick tock, tick tock.')
rawlen = length(raw);
timecol = cell(rawlen,1);
numcols = zeros(rawlen,1);
out = cell(rawlen,1);

for i = 1:rawlen
   temp = raw{i};
   tabs = find(temp == 9);
   numcols(i) = length(tabs); 
   timecol{i} = raw{i}(1:tabs(1)-1);
   out{i,:} = raw{i}(tabs(1): tabs(end)-1);
end

% check num of entries in each line, because number of channels
% can change between subtrials.
temp = numcols(1:end-1)-numcols(2:end);
chan_chg = find(temp~=0) + 1;
if chan_chg
    disp(['The number of channels changed following trial(s): ' num2str(chan_chg)])    
    blockstarts = [1 chan_chg];
    blockstops  = [chan_chg-1 rawlen];
    
    for j = 1:length(blockstarts)
       block{j} = cell2mat(out( blockstarts(j):blockstops(j) ,:));
   end
else
    block{1} = cell2mat(out);
    blockstarts = 1; 
    blockstops = rawlen;
end
numblocks=length(block);
disp('')

cd(pn)
files = 0;
for z = 1:length(block)
   temp = block{z};
   out = str2num(temp); % because str2double gives 'NaN'

   % display the number of channels, ask whether user wants to view them
   % or simply enter/verify channel order.
   [numsamps,numcols] = size(out);
   disp(' ')
   disp(['Block ' num2str(z) ' of ' num2str(numblocks) ])
   disp([ '  ' num2str(numcols) ' channels detected.'])
   disp([ '  ' num2str(numsamps) ' samples detected.'])

   % use slower str2double because cell2mat chokes when 
   % 6-digit time becomes 7-digit time during recording session.
   times_str = timecol(blockstarts(z):blockstops(z),1);
   t=zeros(size(times_str,1),size(times_str,2));
   t=str2double(times_str);
   % For multiple records in a single EDF file, there will be gaps in time between 
   % each experiment. Use them to separate the experiments.
   tdiff=t(2:end) - t(1:end-1);
   filestops = find(tdiff > 100 )';
   filestops =  [ filestops numsamps ];
   filestarts = [ 1 filestops(1:end-1)+1];
   numfilestops = length(filestops);

   if isempty(filestops) || length(filestops)==1
      % if there are no record separator lines, just
      % use the last row of the data as the end point
      disp( '  Only 1 trial detected.' )
      filestarts = 1;
      filestops = numsamps;
   else    
      disp([ '  ' num2str(numfilestops) ' trials detected.'])
      disp([ '  Separations at lines: ' num2str(filestops)] )
      septrials = 'y';
      %septrials = input('  Treat as individual trials? (y/n) ','s');
      if  strfind( septrials, 'y' )
         filestarts = [1, (filestops+1)];
      else
         filestarts = 1; 
         filestops = numsamps;
      end
   end

   % default chan vals coming FROM edf2asc
   %lh_chan=1; lv_chan=2; rh_chan=4; rv_chan=5;
   
   numfilestops = length(filestops);
   for x = 1:numfilestops
      clear rh_chan rv_chan lh_chan lv_chan
      disp(' ')
      disp( [' Record ' num2str(files+x)] )
      disp( ['  Starting time: ' start_time{x}(1:end-1) ] )
      disp( ['  Sampling frequency: ' num2str( sf(files+1) ) ])
      switch numcols
        case 7
          disp( 'Default EDF->ASC export assumption: ' )
          disp( '   1) time, 2) lh, 3) lv, 4) lp (pupil), 5) rh, 6) rv, 7) rp' )
          disp( '   Will save in this order: [lh rh lv rv]' )
          ch_err_flag=1;
          yorn = input('Is this correct? (y/n) ','s');
          if strcmpi(yorn,'y')
             lh_chan=2; lv_chan=3; rh_chan=5; rv_chan=6;
             ch_err_flag = 0;
          end

        case 6
          %disp( 'Default EDF->ASC export assumption: ' )
          %disp( '   1) lh, 2) lv, 3) lp (pupil), 4) rh, 5) rv, 6) rp' )
          disp( '  Will save in this order: [lh rh lv rv]' )
          ch_err_flag=1;
          %yorn = input('Is this correct? (y/n) ','s');
          yorn='y';
          if strcmpi(yorn,'y')
             lh_chan=1; lv_chan=2; rh_chan=4; rv_chan=5;
             ch_err_flag = 0;
          end

        case 5
          disp( 'Default EDF->ASC export assumption: ' )
          disp( '   1) lh, 2) lv, 3) lp (pupil), 4) rh, 5) rv' )
          disp( '   Will save in this order: [lh rh lv rv]' )
          ch_err_flag=1;
          yorn = input('Is this correct? (y/n) ','s' );
          if strcmpi(yorn,'y')
             lh_chan=1; lv_chan=2; rh_chan=4; rv_chan=5;
             ch_err_flag = 0;
          end

        case 3
          ch_err_flag = 1;
          if strcmpi( eyes{files+1}, 'l' )
             lh_chan=1; lv_chan=2;
             disp(['  Left eye only.'])
             disp( '  Will save in this order: [ lh lv ]')
          elseif strcmpi( eyes{files+1}, 'r')
             rh_chan=1; rv_chan=2;
             disp(['  Right eye only.'])
             disp( '  Will save in this order: [ rh rv ]')
          end
          ch_err_flag = 0;

        otherwise
          disp( 'I do not know the order of the channels here.' )
          ch_err_flag = 1;
          clear rh_chan rv_chan lh_chan lv_chan
       end %switch numcols

       % If none of the known cases exist, prompt the user for the channel names.
       % might need this if the data were taken in an unusual form, e.g., monocularly.
       strarray = [ {'lh'},{'rh'},{'lv'},{'rv'},{'lp'},{'rp'} ];
       % name the channels
       if ch_err_flag
          sampfreq = sf(files+1);
          for i=1:numcols
             chtemp = out(:,i); t=maket(chtemp,sampfreq);
             figure; plot(t,chtemp)
             commandwindow
             chname{i} = input(['Enter a name for channel ' num2str(i) '.  Enter "-" to ignore it: '],'s');
             chpos = strcmp(chname{i}, strarray);
             if chpos
               eval( [strarray{chpos} '_chan = i;'])
               %disp(['   Assigning channel ' str2num(i) ' as ' chname{i} '.'] )
             end   
          end
       end

       % time to write out to file(s)
       clear temp
       dat = [];
       if exist('lh_chan','var')  dat =        out(filestarts(x):filestops(x),lh_chan);  end
       if exist('rh_chan','var'), dat = [dat,  out(filestarts(x):filestops(x),rh_chan)]; end
       if exist('lv_chan','var'), dat = [dat, -out(filestarts(x):filestops(x),lv_chan)]; end
       if exist('rv_chan','var'), dat = [dat, -out(filestarts(x):filestops(x),rv_chan)]; end
       temp{x} = [fname '_' num2str(files+x) '.bin'];       
       fid = fopen(temp{x}, 'w', 'l');
       fwrite(fid, dat, 'float');
       fclose(fid);
       disp(['  Saved as ' '''' pn temp{x} ''''])
     end
     files = files + length(filestops);
        
end % for z

% write the starting times to a text file
fid = fopen( [fname '_start_times.txt'], 'w', 'l');
fwrite(fid, cell2mat(start_time), 'char');
fclose(fid);

try
   cd(curdir)
 catch
   cd(matlabroot)
end  

disp(' ')
disp('Remember that you will need an adjustbias file to read the data using ''rd''.')
disp('Type ''biasgen'' to create the adjustbias file.   You will need to know the')
disp('sampling frequency used to take the data, as well as which channels were recorded.')
disp('When ''biasgen'' asks, the Method is (V)ideo and the Format is (B)inary,')
disp('the Calibration is (N)ormal and the Channel Structure is (C)ontiguous.')