% desacc.m: Interactive GUI front end that calls 'findsaccs' and displays its
% output in dynamically updated window.
%
% each time  the display updates, the following variables, containing the results,
% are written to the base workspace:
%    pos_desacc -- the position, desaccaded
%    vel_desacc -- the velocity, desaccaded
%
% 'findsaccs' saves these variables into the base workspace:
%    ptlist     -- the vector of 'yes'/'no' points (good data = 1, sacc = 0 or NaN)
%    pvlist     -- the samp # where each PV occurred in the fast phases
%    saccstart  -- start point of each saccade
%    saccstop   -- end point of each saccade
%    pvel       -- the values of the peak velocities of the fast phases
%    extend     -- the value of the 'extend' setting
%    dataName   -- the string containing the name of the data being analyzed
%    thresh_a   -- the threshold (accel) above which saccades is considered "ON"
%    thresh_v   -- the threshold (vel) above which saccades is considered "ON"
%    acc_stop   -- the threshold (accel) below which saccades is considered "ON"
%    vel_Stop   -- the threshold (vel) below which saccades is considered "ON"
%
% Requires: 'findsaccs.m', 'd2pt.m', 'maket.m', 'stripnan.m', 'zoomtool' suite

% written by:  Jonathan Jacobs
%              September 2003 - May 2004 (last mod: 05/27/04)

function null = desacc(action)

% look for the open windows
sacc_cont = -1;
wl = get(0,'Children');
for i = 1:length(wl)
   if strcmp(get(wl(i),'Name'),'Desaccade Control'), sacc_cont=wl(i);  end
end

if nargin == 0 & sacc_cont>0
   figure(sacc_cont)
   return
end

if sacc_cont<0
   action= 'initialize';
 else
   % retrieve the control handles
   contH = get(sacc_cont,'UserData');
   strict_stripH = contH{1};  dataNameH     = contH{2};
   vel_or_accH   = contH{3};  thresh_a_onH  = contH{4};
   gap_fpH       = contH{5};  gap_spH       = contH{6};
   sacc_result   = contH{7};  knockoutH     = contH{8};
   extendH       = contH{9};  doneH         = contH{10};
   thresh_v_onH  = contH{11}; thresh_a_offH = contH{12};
   thresh_v_offH = contH{13};
end


%%%%%%%%%%%%%%%%%%%%%%  initialization  %%%%%%%%%%%%%%%%%%%%%%
if strcmp(action,'initialize')

   fig_ht = 240+30; fig_wid = 275;
   
   % read the last controls settings from the pref file.  if not present
   % or if nonsensical, set controls to default settings.
	% if there is no 'omprefs' folder in the MATLAB root, we will
	% create one and also create a prefs file.
	cur_dir = pwd;
	cd(matlabroot)
	gp_err=0;
	eval('cd(findomprefs)','gp_err=1;')
	if gp_err % must make a omprefs directory
		mkdir('omprefs')
		cd('omprefs')
	end
	
	% default pref values
	knockout      = 2;
	vel_or_acc    = 1;
	strict_strip  = 1;
	thresh_a      = '800';
	acc_stop      = '100';
	thresh_v      = '20';
	vel_stop      = '10';
	extend        = '0';
    gap_fp		  = '10';
    gap_sp		  = '10';
	cwind_pos     = [100 100 fig_wid fig_ht];
	
	% if there is not a good pref file, create one w/the default values
	pref_err=0;
	eval('load(''desaccPrefs.mat'')','pref_err=1;')   
	if pref_err
		disp('pref_err!')
		save desaccPrefs.mat strict_strip knockout vel_or_acc ...
		                     thresh_a thresh_v acc_stop vel_stop ...
		                     extend gap_fs gas_sp cwind_pos
	end
	if exist(cur_dir,'dir'), cd(cur_dir); end


	sacc_cont = figure('Position',cwind_pos, ...
		 'Name','Desaccade Control','NumberTitle','off', 'Resize', 'off', ...
		 'Menubar','none','Tag','desaccade_control: uneditable');
		 %'color','k' );
	
	uicontrol(sacc_cont, 'Style','Frame','Units','Pixels',...
		'BackgroundColor',[0.25 0.25 0.25],'Position',[2 2 fig_wid-4 fig_ht-4])
	
	x_origin = 0;  y_origin = fig_ht;
	xpos = 5;      ypos = y_origin - 30;
	
	uicontrol(sacc_cont, 'Style', 'text',...
		'Position', [xpos ypos 145 20], 'String', 'Mark saccade as:');
	knockoutH = uicontrol(sacc_cont, 'Style', 'popup', 'Units', 'pixels',...
		 'Position',[xpos+147 ypos 123 20],...
		 'String','zeros|NaNs|delete|untouched',...
		 'Tooltip','Mark saccades as zeros or NaNs',...
		 'HorizontalAlignment', 'center', 'Value', knockout,... 
		 'Callback',['desacc(''calculate'');']);

	ypos=ypos-25;
	uicontrol(sacc_cont, 'Style', 'text',...
		 'Position', [xpos ypos 145 20], 'String', 'Data name');
	dataNameH = uicontrol(sacc_cont, 'Style','edit','Units','pixels',...
		 'BackgroundColor','magenta','ForeGroundColor','white',...
		 'Position',[xpos+150 ypos 115 20],'String', '',...
		 'Tooltip',['Enter the name of the workspace' char(13) ...
		            'variable containing the data'],...
		 'Callback',['desacc(''calculate'');']);
	
	ypos=ypos-25;
	uicontrol(sacc_cont, 'Style', 'text',...
		'Position', [xpos ypos 145 20], 'String', 'Detect by:');
	vel_or_accH = uicontrol(sacc_cont, 'Style', 'popup', 'Units', 'pixels',...
		 'Position',[xpos+147 ypos 123 20],...
		 'String','Acceleration|Velocity|Both',...
       'Visible','on',...
		 'Tooltip',['Detect saccades using' char(13) 'velocity and/or acceleration?'],...
		 'HorizontalAlignment', 'center', 'Value', vel_or_acc,... 
		 'Callback',['']);


	ypos=ypos-25;
	uicontrol(sacc_cont, 'Style', 'text',...
		 'Position', [xpos ypos 145 20], 'String', 'Fast Phase Removal:');
	strict_stripH = uicontrol(sacc_cont, 'Style', 'popup', 'Units', 'pixels',...
		 'Position',[xpos+147 ypos 123 20],...
		 'String','Strict|Loose',...
		 'Tooltip',['Strict: remove only "definite"' char(13) ...
		            '(i.e. PV>Vel Thresh) saccades.' char(13) ...
		            'Loose: Remove all points that ' char(13) ...
		            'exceed the Acc/Vel thresholds'],...
		 'HorizontalAlignment', 'center', 'Value', strict_strip,... 
		 'Callback',['']);
		 %'Callback',['desacc(''calculate'');']);
	
	ypos=ypos-25;
	uicontrol(sacc_cont, 'Style', 'text',...
		 'Position', [xpos ypos 145 20], 'String', 'Acc Thresh (on,off)');
	thresh_a_onH = uicontrol(sacc_cont, 'Style','edit','Units','pixels',...
		 'BackgroundColor','magenta','ForeGroundColor','white',...
		 'Position',[xpos+150 ypos 50 20],'String', thresh_a,...
		 'Tag','thresh_text', ...
		 'Tooltip',['Acc value above which the' char(13) ...
		            'saccade is considered' char(13) ...
		            'to have turned "ON"'],...
		 'Callback',['desacc(''calculate'');']);

	thresh_a_offH = uicontrol(sacc_cont, 'Style','edit','Units','pixels',...
		 'BackgroundColor','magenta','ForeGroundColor','white',...
		 'Position',[xpos+200 ypos 50 20],'String', acc_stop,...
		 'Tag','acc_stop', ...
		 'Tooltip',['Acc value below which the' char(13) ...
		            'saccade is considered' char(13) ...
		            'to have turned "OFF"'],...
		 'Callback',['desacc(''calculate'');']);

	ypos=ypos-25;
	uicontrol(sacc_cont, 'Style', 'text',...
		'Position', [xpos ypos 145 20], 'String', 'Vel Thresh (on,off)');
	thresh_v_onH = uicontrol(sacc_cont, 'Style', 'edit', 'Units', 'pixels',...
		 'BackgroundColor','magenta','ForeGroundColor','white',...
		 'Position',[xpos+150 ypos 50 20],'String',thresh_v,...
		 'Tooltip',['Vel value above which the' char(13) ...
		            'saccade is considered' char(13) ...
		            'to have turned "ON".'  char(13) ...
		            '(Always used, even when' char(13) ...
		            '"Detect" is accel only.)'],...
		 'HorizontalAlignment', 'center',... 
		 'Callback',['desacc(''calculate'');']);
	
	thresh_v_offH = uicontrol(sacc_cont, 'Style','edit','Units','pixels',...
		 'BackgroundColor','magenta','ForeGroundColor','white',...
		 'Position',[xpos+200 ypos 50 20],'String', vel_stop,...
		 'Tag','acc_stop', ...
		 'Tooltip',['Vel value below which the' char(13) ...
		            'saccade is considered' char(13) ...
		            'to have turned "OFF"'],...
		 'Callback',['desacc(''calculate'');']);
	
	ypos=ypos-30;
	uicontrol(sacc_cont, 'Style', 'text',...
		'Position', [xpos ypos 145 20], 'String', 'Extend by (samps):');
	extendH = uicontrol(sacc_cont, 'Style','edit','Units','pixels',...
		 'BackgroundColor','magenta','ForeGroundColor','white',...
		 'Position',[xpos+150 ypos 60 20],'String', extend,...
		 'Tooltip',['Extend the deletion before' char(13) ...
		            'and after the saccade by' char(13) ...
		            'this many samples'],...
		 'Callback',['desacc(''calculate'');']);

	ypos=ypos-25;
	uicontrol(sacc_cont, 'Style', 'text',...
		'Position', [xpos ypos 145 20], 'String', 'Max Fast Ph. gap size');
   gap_fpH = uicontrol(sacc_cont, 'Style','edit','Units','pixels',...
		 'BackgroundColor','magenta','ForeGroundColor','white',...
		 'Position',[xpos+150 ypos 60 20],'String', gap_fp,...
		 'Tooltip',['Number of sub-threshold' char(13) ... 
		            'samples allowed when' char(13) ...
		            'finding fast phase segments'],...
		 'Callback',['desacc(''calculate'');']);

	ypos=ypos-25;
	uicontrol(sacc_cont, 'Style', 'text',...
		'Position', [xpos ypos 145 20], 'String', 'Max Slow Ph. gap size');
	gap_spH = uicontrol(sacc_cont, 'Style','edit','Units','pixels',...
		 'BackgroundColor','magenta','ForeGroundColor','white',...
		 'Position',[xpos+150 ypos 60 20],'String', gap_sp,...
		 'Tooltip',['Number of supra-threshold' char(13) ...
		            'samples allowed when' char(13) ...
		            'finding slow phase segments'],...
		 'Callback',['desacc(''calculate'');']);
	
	ypos=ypos-25;
	helpH = uicontrol(sacc_cont, 'Style', 'Pushbutton',...
		 'Position', [xpos ypos 40 20],'UserData',0,...
		 'Tooltip','Help me, Spock!',...
		 'String', 'Help',...
		 'Callback', ['help desacc'] );

	
   aboutStr = {'desacc -- Version 1.0,  May 2004';
               ' ';
               'Written by Jonathan Jacobs, Ph.D. (jxj24@cwru.edu)';
               ' ';
               'Ocular Motor Neurophysiology Lab (www.OMLAB.org)';
               'VA Medical Center, Cleveland, OH';
               ' ';
               'Dept. of Neurology, School of Medicine';
               'Case Western Reserve University';
               ' ';
               'All rights reserved.  All wrongs reversed.';
               '(copyleft) 2002 - 2004';
               ' '};

	aboutH = uicontrol(sacc_cont, 'Style', 'Pushbutton',...
		 'Position', [xpos+50 ypos 50 20],'UserData',0,...
		 'Tooltip','Whodunnit?',...
		 'String', 'About...', 'UserData', aboutStr, ...
		 'Callback', ['msgbox(get(gco,''UserData''),''About "desacc"'')'] );

	doneH = uicontrol(sacc_cont, 'Style', 'Pushbutton',...
		 'Position', [xpos+150 ypos 50 20],'UserData',0,...
		 'Tooltip','Done',...
		 'String', 'Done',...
		 'Callback', ['desacc(''done'');'] );


	%%%%%%%% results window %%%%%%%
	sacc_result = figure('Position',[340   171   704   524], ...
		 'Name',['Desaccading Monitor'],'NumberTitle','off', ...
		 'Menubar','none','Tag','desaccade_monitor'); %,...
		 %'color','k' );
	
	figure(sacc_result)
	line([0 10 100 1000 10000 1000000],[0 0 0 0 0 0])
	set(doneH,'UserData', gca)
	box
	zoomtool

   % store all the handles in the userdata of the figure window
   % so we can access them when we calculate (or perform other 
   % non-init actions).
   contH = {strict_stripH   dataNameH    vel_or_accH    thresh_a_onH   ...
            gap_fpH         gap_spH      sacc_result    knockoutH      ...
            extendH		    doneH	     thresh_v_onH   thresh_a_offH  ...
            thresh_v_offH};
            
   set(sacc_cont,'Userdata', contH)
	figure(sacc_cont)


 %%%%%%%%%%%%%%%%%%%%%%  calculation  %%%%%%%%%%%%%%%%%%%%%%
 elseif strcmp(action,'calculate')
   thresh_a     = str2num(get(thresh_a_onH,'string'));
   thresh_v     = str2num(get(thresh_v_onH,'string'));
   acc_stop     = str2num(get(thresh_a_offH,'string'));
   vel_stop     = str2num(get(thresh_v_offH,'string'));
   gap_fp       = str2num(get(gap_fpH,'string'));
   gap_sp       = str2num(get(gap_spH,'string'));
   extend       = str2num(get(extendH,'string'));
   vel_or_acc   = get(vel_or_accH,'value');
   ko_val       = get(knockoutH,'value');
   strict_strip = get(strict_stripH,'value');
   dataName     = get(dataNameH,'String');
   
diff_level = 5;
   
   if isempty(dataName), return; end
   switch ko_val
      case 1, knockout = 'zeros';
      case 2, knockout = 'NaNs';
      case 3, knockout = 'delete';
      case 4, knockout = 'untouched';
      otherwise disp('knockout is unknown')
   end
   
   % turn 'dataName' into actual data by evaluating in base workspace
   % we create a temporary global variable that can be accessed in both
   % base and function workspaces.   
	global tempresult
	evalin('base','global tempresult')
   
	tempresult = 0;
	% Make sure that the data we want to use really exists
	evalin('base',['tempresult = exist(' '''' dataName '''' ',''var'');'],...
					  ['disp(''error'')'])
	% copy 'tempresult' into 'in' which will be passed to 'findsaccs'

	eval_err = 0;
	evalin('base',['tempresult = ' dataName ';'],['eval_err = 1;'])
   if eval_err
      disp(['There is no variable named ' dataName ' in the workspace.']);
      set(dataNameH,'String','')
      return;
   end
	pos = tempresult;	
	t=maket(pos);
	clear global tempresult

  
   % call 'findsaccs' and update the results window
   % NOTE: 'in' will be converted to vel or acc in 'findsaccs'
   [ptlist, pvlist] = findsaccs(pos, thresh_a, thresh_v, ...
                      acc_stop, vel_stop, gap_fp, gap_sp, ...
                      vel_or_acc, extend, dataName, strict_strip);
   
	% ptlist comes out of 'findsacc' with bad pts as '0', but we might want 'NaNs'
   if strcmp(knockout(1),'z')
      ;
    elseif strcmp(knockout(1),'u')
     ptlist = ones(length(ptlist),1);
    else
     % done for 'NaN' and 'delete' popup choices
     tempnan = NaN*ones(length(ptlist),1);
     nan_pts = find(ptlist == 0);
     ptlist(nan_pts) = tempnan(nan_pts);
	end
	
   % for fun, let's have a complement to the list of good points (ptlist)
   badpts = NaN*zeros(size(ptlist));
   temp   = find(isnan(ptlist));
   badpts(temp) = 1;
 
	% convert input array from position to either vel or accel.
	vel = d2pt(pos,diff_level);
	acc = d2pt(vel,diff_level);

	pos_d = pos .* ptlist;
	vel_d = vel .* ptlist;
	acc_d = acc .* ptlist;
 
	posbad_d = pos .* badpts;
	velbad_d = vel .* badpts;
	accbad_d = acc .* badpts;

	% prep 'vel', 'pos' arrays for display with the accel array (if necessary)
   pscale = mean(abs(stripnan(acc)))/mean(abs(stripnan(pos)))*3;
   pscale = fix(pscale);
   vscale = mean(abs(stripnan(acc)))/mean(abs(stripnan(vel)));
   vscale = fix(vscale);
   
	% if knockout mode is 'delete' use stripnan on ptlist
   if strcmp(knockout(1),'d')
      pos_d    = stripnan(pos_d);
      vel_d    = stripnan(vel_d);
      acc_d    = stripnan(acc_d);
      t        = stripnan(t .* ptlist);
      ptlist   = stripnan(ptlist);
      posbad_d = stripnan(posbad_d);
      velbad_d = stripnan(velbad_d);
   end	

   % display
   figure(sacc_result)
   set(sacc_result,'Name','Desaccading Monitor','NumberTitle','Off')
   zoomclr( get(sacc_result,'CurrentAxes') )
   % would like to keep axis limits same
   % read them and then set them
   currAxes=get(sacc_result,'CurrentAxes');
   xlims = get(currAxes,'Xlim');
   clf
   hold on
   pos_lineH = plot(t,pscale*pos_d,'y');
   if ~strcmp(knockout(1),'u')
      posbad_lineH = plot(t,pscale*posbad_d,'color',[0.5 0.5 0],'vis','on'); %dark yellow
   end
   vel_lineH = plot(t,vscale*vel_d,'g');
   if ~strcmp(knockout(1),'u')
      velbad_lineH = plot(t,vscale*velbad_d,'color',[0 0.4 0],'vis','on');   %dark green
   end
   
   if length(pvlist), pvpt_line = plot(t(pvlist),vscale*vel(pvlist),'mo'); end
   acc_lineH = plot(t,acc, 'r');
   accthresh_lineH = plot([t' NaN t'],[ptlist'*thresh_a NaN -ptlist'*thresh_a], 'b');
   velthresh_lineH = plot([t' NaN t'],...
                    [ptlist'*thresh_v*vscale NaN -ptlist'*thresh_v*vscale], 'c');
   xlabel('Time (sec)')
   ylabel('r: /s^2, g: /s (scaled), y:  (scaled)')
   box
   set(gca,'Xlim',xlims);
   set(doneH,'UserData',gca)
   title(['r: acc, g: vel, y: pos, b: y/n (acc), c: y/n (vel), m: PV' char(13) ...
          'pscale: ' num2str(pscale) ', vscale: ' num2str(vscale) ...
          ', vthresh: ' num2str(thresh_v) ', athresh: ' num2str(thresh_a)])
   zoomtool 
   
   % the following variables were assigned to the base workspace in 'findsaccs':
   %   saccstart,  saccstop
   %   ptlist,      pvlist,      pv,         extend.
   % Use them in good health.
   assignin('base','pos_desacc', pos_d);  
   assignin('base','vel_desacc', vel_d);  

 %%%%%%%%%%%%%%%%%%%%%%  clean up %%%%%%%%%%%%%%%%%%%%%%
 elseif strcmp(action,'done')

   cwind = get(gco,'Parent');
   
   % read the controls values and save to an pref file
   strict_strip  = get(strict_stripH, 'Value');
   knockout      = get(knockoutH, 'Value');
   vel_or_acc    = get(vel_or_accH, 'Value');
   thresh_a      = get(thresh_a_onH, 'String');
   thresh_v      = get(thresh_v_onH, 'String');
   acc_stop      = get(thresh_a_offH, 'String');
   vel_stop      = get(thresh_v_offH, 'String');
   extend        = get(extendH, 'String');
   cwind_pos     = get(cwind, 'Position');
   gap_fp        = get(gap_fpH,'string');
   gap_sp        = get(gap_spH,'string');
  

   temp=findwind('Desaccading Monitor');
   if temp > 0
      figure(temp)
      set(temp,'Name', 'Desaccaded result')
      set(temp,'Tag', '')
      zoomclr
   end
   close(cwind)

   cur_dir=pwd;
   %cd(matlabroot)
   cd(findomprefs)
   save desaccPrefs.mat strict_strip knockout vel_or_acc ...
                        thresh_a thresh_v acc_stop vel_stop ...
                        extend gap_fp gap_sp cwind_pos
   eval('cd(cur_dir)','')
  
 else
   disp(['desacc: unknown action -- ' action])
   
end